##################################################################
##  
##  Aseryla project [http://aseryla.16mb.com]
##  Information Extraction task
##
  

import sys      #
import time     # 
import socket   # aseryla connections


aserPort = 9901 
nlpPort = 9902 
 
 
 
 
def appendFile(content, filename):
    f = open(filename, 'a')
    f.write(content)
    f.close()
 

 
## Aseryla server communication ###################################################################################################################
 
EndCommSymbol = "\b"
 
# manages a socket read message
def socket_receive(sock):
    total_data=[]
    data=''
    while True:
            data=sock.recv(8192)
            if EndCommSymbol in data:
                total_data.append(data[:data.find(EndCommSymbol)])
                break
            total_data.append(data)
            if len(total_data)>1:
                #check if end_of_data was split
                last_pair=total_data[-2]+total_data[-1]
                if EndCommSymbol in last_pair:
                    total_data[-2]=last_pair[:last_pair.find(EndCommSymbol)]
                    total_data.pop()
                    break
    return ''.join(total_data)
	
 
	
 
 
# send a message to the Aseryla server, and reads the response
def send_process(order, parameter, port):
     
    try:
        # connect with the server
        sock = socket.socket()
        sock.connect(("localhost", port))
 
        #send the entire message with the \b end symbol at the end (the expected final symbol in the server)
        sock.sendall(" " + order + " " + parameter + EndCommSymbol)
 
        # ok = the results list 
        # K0 = no content
        # K1 = no nlp
        # K2 = error process
        # K3 = no server
 
        # read the response
        res = socket_receive(sock)
 
        # close the communication    
        sock.close()
 
        return res[1:-1]
    except:
        return "K3"
 
 
 
 


   
# process a bunch of sentences
def process_text(text):

##       K0 if not relations has been identified or if the input is not a valid sentence (filtered in the lexical part)
##       K1 if the NLP server is not ready
##       K2 in case any processing error
##        a newline separated list with the ILCs    
    return send_process("Ts", text, aserPort)
 

     
# checks if the Aseryla server is running properly
def check_aser_server():
    return send_process("@h", "", aserPort) != 'K3'
  

# checks if the nlp server is running properly
def check_nlp_server():
    res = send_process("@n", "", aserPort)
    
    return res != "K3" and res[0] != '0'
  


 
 
 
 
####################################################################################################################
##  MAIN  ##########################################################################################################
####################################################################################################################
 

good = True
if not check_aser_server():
    print "NO SERVER"
    good = False
    

if not check_nlp_server():
    print "NO NLP"
    good = False

if good:    


    send_process("@w", "0", aserPort) # disable autosave mechanism
        
    currents = []

    with open("key_matched.txt") as f: 
        for line in f:
            if line[0:2] == "**":

                currents = list(set(currents))

                if len(currents) > 0:
                    appendFile("\n" + "\n".join(currents), "final_ilcs.txt")
                    currents = []
                
            else:
                res1 = process_text(line.strip())

                res = res1[0:2]


                if res == "K3":
                    print "Aseryla server down"
                    break
                else:
                    if res == "K1":
                        print "NLP server is not ready"
                        break
                    else:
                        if res == "K2":
                            print "processing error"
                            break
                        else:
                            if res != "K0":
                                currents.extend(res1.split("\n"))


    currents = list(set(currents))
    if len(currents) > 0:
        appendFile("\n" + "\n".join(currents), "final_ilcs.txt")

                    
    send_process("@v", "0", aserPort) # disable autosave mechanism  


    print "end"

